#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Jussi Määttä, 2014

# Construct a Huffman code for a given input and output it in JSON format.
#
# Usage:
#
#     build_codebook.py < input > codebook.json

import sys
import string
import heapq
import json

freq = {}

for char in sys.stdin.read():
    if char in freq:
        freq[char] += 1
    else:
        freq[char] = 1

heap = []
for (key, value) in freq.items():
    heapq.heappush(heap, (value, key))

children = {}

while len(heap) >= 2:
    (x_count, x_char) = heapq.heappop(heap)
    (y_count, y_char) = heapq.heappop(heap)
    z_count = x_count + y_count
    z_char = x_char + y_char
    children[z_char] = (x_char, y_char)
    heapq.heappush(heap, (z_count, z_char))

(_, root) = heapq.heappop(heap)

codebook = {}
stack = [ (root, '') ]
while len(stack) > 0:
    (x, s) = stack.pop()
    if children.get(x) == None:
        codebook[x] = s
    else:
        stack.append((children[x][0], s + '0'))
        stack.append((children[x][1], s + '1'))

print(json.dumps(codebook))