% Jussi Maatta 2014

%
% Initialization
%
close all;
clear;

%
% Parameters
%
n = 100;
q = 0.1;
p_limit = 0.93;

%
% Find the largest k such that buying all tickets with at most k ones
% will still leave us below the desired winning probability p_limit.
%
p_total = 0;
k = 0;
while true
    % The line below is equivalent to
    %     p = nchoosek(n, k) * q^k * (1-q)^(n-k)
    % but has better numerical precision.
    p = binopdf(k, n, 0.1);
    
    if (p_total + p) >= p_limit
        break;
    else
        p_total = p_total + p;
        k = k + 1;
    end
end
k = k - 1;

%
% Calculate the number of tickets with at most k ones.
% (This step could have been combined with the previous one,
% but this is probably more numerically stable. Better still,
% we could compute the sum in log-domain.)
%
count_1 = 0;
for i = 0:k
    count_1 = count_1 + nchoosek(n, i);
end
fprintf('First, buy all tickets with at most %d ones.\n', k);
fprintf('There are about %.4e such tickets.\n', count_1);

%
% Calculate an approximation of how many tickets with k+1 ones are needed.
%
count_2 = (p_limit - p_total) / (q^(k+1)) / ((1-q)^(100-(k+1)));
fprintf('Then, buy about %.4e tickets with exactly %d ones.\n', ...
    count_2, k+1);

%
% Conclusion.
%
fprintf('All in all, we buy about %.2e tickets.\n', count_1 + count_2);